__title__ = 'vishap.exceptions'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('PluginCodeError', 'ImproperlyConfigured', 'PluginNotFound', 'PluginDetectionError', \
           'InvalidRegistryItemType')

class PluginCodeError(Exception):
    """
    Exception raised when language code is left empty or has incorrect value.
    """

class ImproperlyConfigured(Exception):
    """
    Exception raised when developer didn't configure the code properly.
    """

class PluginNotFound(Exception):
    """
    Exception raised when language pack is not found for the language code given.
    """

class PluginDetectionError(Exception):
    """
    Exception raised when language can't be detected for the text given.
    """

class InvalidRegistryItemType(ValueError):
    """
    Raised when an attempt is made to register an item in the registry which does not have a proper type.
    """


__title__ = 'vishap.contrib.plugins.vimeo.vishap_plugin'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('VimeoPlugin',)

from vishap.base import BaseVideoPlugin, plugin_registry

class VimeoPlugin(BaseVideoPlugin):
    """
    Vimeo plugin.
    """
    uid = "vimeo"
    name = "Vimeo"
    url_pattern = "^(?P<prefix>(http(s)?\:\/\/www\.vimeo\.com\/)|(http(s)?\:\/\/vimeo\.com\/))(?P<value>\d*)"
    id_pattern = "^(?P<value>\d*{11})"
    embed_code = """
    <iframe src="//player.vimeo.com/video/{video_id}" {options} frameborder="0" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>
    """

plugin_registry.register(VimeoPlugin)



__title__ = 'vishap.conf'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('get_setting', 'set_setting', 'settings', 'reset_to_defaults_settings')

from vishap import defaults

class Settings(object):
    """
    Settings registry.
    """
    def __init__(self):
        self._settings = {}

    def set(self, name, value):
        """
        Override default settings.

        :param str name:
        :param mixed value:
        """
        self._settings[name] = value

    def get(self, name, default=None):
        """
        Gets a variable from local settings.

        :param str name:
        :param mixed default: Default value.
        :return mixed:
        """
        if name in self._settings:
            return self._settings.get(name, default)
        elif hasattr(defaults, name):
            return getattr(defaults, name, default)
        else:
            return default

    def reset_to_defaults(self):
        """
        Resets settings to defaults.
        """
        self._settings = {}

settings = Settings()

get_setting = settings.get

set_setting = settings.set

reset_to_defaults_settings = settings.reset_to_defaults



__title__ = 'vishap.discover'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('autodiscover',)

import os
import logging

try:
    from importlib import import_module
except ImportError:
    import_module = __import__

from vishap.helpers import PROJECT_DIR
from vishap.conf import get_setting

logger = logging.getLogger(__name__)

def autodiscover():
    """
    Autodiscovers the plugins in contrib/plugins.
    """
    PLUGINS_DIR = get_setting('PLUGINS_DIR')
    PLUGIN_MODULE_NAME = get_setting('PLUGIN_MODULE_NAME')

    for app_path in os.listdir(PROJECT_DIR(PLUGINS_DIR)):
        full_app_path = list(PLUGINS_DIR)
        full_app_path.append(app_path)
        if os.path.isdir(PROJECT_DIR(full_app_path)):
            try:
                import_module(
                    "vishap.{0}.{1}.{2}".format('.'.join(PLUGINS_DIR), app_path, PLUGIN_MODULE_NAME)
                    )
            except ImportError as e:
                logger.debug(str(e))
            except Exception as e:
                logger.debug(str(e))
        else:
            pass



__title__ = 'vishap.helpers'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('PROJECT_DIR',)

import os

PROJECT_DIR = lambda base: os.path.abspath(
    os.path.join(
        os.path.dirname(__file__),
        (os.path.join(*base) if isinstance(base, (list, tuple)) else base)
    ).replace('\\','/')
)

#PROJECT_DIR = lambda base : os.path.abspath(os.path.join(os.path.dirname(__file__), base).replace('\\','/'))


__title__ = 'vishap.base'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BaseVideoPlugin', 'plugin_registry')

import re

from six import string_types

from vishap.exceptions import ImproperlyConfigured, InvalidRegistryItemType

mark_safe = lambda s: s

class BaseVideoPlugin(object):
    """
    Base video plugin.
    """
    uid = None
    name = None
    description = None
    url_pattern = None
    id_pattern = None
    thumbnail_pattern = None
    embed_code = None
    width_param_code = 'width="{width}"'
    height_param_code = 'height="{height}"'

    def __init__(self):
        # Making sure all necessary properties are defined.
        for prop in ('uid', 'name', 'url_pattern', 'embed_code', 'width_param_code', 'height_param_code'):
            if not getattr(self, prop):
                raise ImproperlyConfigured(
                    "You should define ``{0}`` property in your ``{1}.{2}`` class.".format(
                        prop, self.__class__.__module__, self.__class__.__name__
                        )
                    )

        # Making sure all necessary methods are defined.
        for meth in ('match', 'render'):
            if not callable(getattr(self, meth, None)):
                raise ImproperlyConfigured(
                    "You should define ``{0}`` method in your ``{1}.{2}`` class.".format(
                        prop, self.__class__.__module__, self.__class__.__name__
                        )
                    )
        self.regex = re.compile(self.url_pattern)

    def _match(self, url):
        """
        Generic check if URL matches the pattern.

        :param str url:
        :return bool:
        """
        match = self.regex.match(url)
        return match

    def match(self, url):
        """
        Check if URL matches the pattern.

        :param str url:
        :return bool:
        """
        return self._match(url)

    def _render(self, url, width=None, height=None):
        """
        Renders the HTML code of embed video based on from URL given.

        :param str url:
        :param int width:
        :param int height:
        :return str:
        """
        match = self.match(url)
        if match:
            video_id = match.group('value')
            options = []
            if width:
                options.append(self.width_param_code.format(width=width))
            if height:
                options.append(self.height_param_code.format(height=height))
            response = mark_safe(self.embed_code.format(video_id=video_id, options=' '.join(options)))
            return response

        return ''

    def render(self, url, width=None, height=None):
        """
        Renders the HTML code of embed video based on from URL given.

        :param str url:
        :param int width:
        :param int height:
        :return str:
        """
        return self._render(url=url, width=width, height=height)


class BaseRegistry(object):
    """
    Registry of dash plugins. It's essential, that class registered has the ``uid`` property.
    """
    type = None

    def __init__(self):
        assert self.type
        self._registry = {}
        self._forced = []

    def register(self, cls, force=False):
        """
        Registers the plugin in the registry.

        :param mixed.
        """
        if not issubclass(cls, self.type):
            raise InvalidRegistryItemType("Invalid item type `{0}` for registry `{1}`".format(cls, self.__class__))

        # If item has not been forced yet, add/replace its' value in the registry
        if force:

            if not cls.uid in self._forced:
                self._registry[cls.uid] = cls
                self._forced.append(cls.uid)
                return True
            else:
                return False

        else:

            if cls.uid in self._registry:
                return False
            else:
                self._registry[cls.uid] = cls
                return True

    def unregister(self, cls):
        uid = None
        # Unregister by ``uid``.
        if isinstance(cls, string_types):
            uid = cls
        elif issubclass(cls, self.type):
            uid = cls.uid
        else:
            raise InvalidRegistryItemType("Invalid item type `{0}` for registry `{1}`".format(cls, self.__class__))

        # Only non-forced items are allowed to be unregistered.
        if uid in self._registry and not uid in self._forced:
            self._registry.pop(uid)
            return True
        else:
            return False

    def get(self, uid, default=None):
        """
        Gets the given entry from the registry.

        :param str uid:
        :return mixed.
        """
        return self._registry.get(uid, default)


class PluginRegistry(BaseRegistry):
    """
    Plugin registry.
    """
    type = BaseVideoPlugin


# Register plugins by calling plugin_registry.register()
plugin_registry = PluginRegistry()



# -*- coding: utf-8 -*-

__title__ = 'vishap.tests'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('VishapTest',)

import unittest
import six
from six import print_

from vishap.conf import set_setting, get_setting, reset_to_defaults_settings
from vishap import defaults
from vishap import get_registered_plugin_uids, render_video
from vishap.base import BaseVideoPlugin, plugin_registry

PRINT_INFO = True
TRACK_TIME = False

def print_info(func):
    """
    Prints some useful info.
    """
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        if TRACK_TIME:
            import simple_timer
            timer = simple_timer.Timer() # Start timer

        result = func(self, *args, **kwargs)

        if TRACK_TIME:
            timer.stop() # Stop timer

        print_('\n{0}'.format(func.__name__))
        print_('============================')
        print_('""" {0} """'.format(func.__doc__.strip()))
        print_('----------------------------')
        if result is not None:
            try:
                print_(result)
            except Exception as e:
                print_(result.encode('utf8'))

        if TRACK_TIME:
            print_('done in {0} seconds'.format(timer.duration))

        return result
    return inner


def py2only(func):
    """
    Skips the test on Python 3.
    """
    if six.PY2:
        return func

    def dummy(self, *args, **kwargs):
        pass

    return dummy


class VishapTest(unittest.TestCase):
    """
    Tests of ``vishap.utils.render_video``.
    """
    def setUp(self):
        self.vimeo_url = 'http://vimeo.com/45655450'
        self.youtube_url = 'http://www.youtube.com/watch?v=LIPl7PtGXNI'

        self.rendered_vimeo_embed_code = """
        <iframe src="//player.vimeo.com/video/45655450" width="500" height="281" frameborder="0" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>
        """.strip()

        self.rendered_vimeo_embed_code_responsive = """
        <iframe src="//player.vimeo.com/video/45655450"  frameborder="0" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>
        """.strip()

        self.rendered_youtube_embed_code = """
        <iframe src="//www.youtube.com/embed/LIPl7PtGXNI" width="560" height="315" frameborder="0" allowfullscreen></iframe>
        """.strip()

        self.rendered_youtube_responsive_embed_code = """
        <iframe src="//www.youtube.com/embed/LIPl7PtGXNI"  frameborder="0" allowfullscreen></iframe>
        """.strip()

        #reset_to_defaults_settings()

    @print_info
    def test_01_get_available_plugin_uids(self):
        """
        Test ``autodiscover`` and ``get_available_plugin_uids``.
        """
        res = get_registered_plugin_uids()
        res.sort()
        c = ['vimeo', 'youtube']
        c.sort()
        self.assertEqual(res, c)
        return res

    @print_info
    def test_02_render_vimeo(self):
        """
        Test rendering of Vimeo.
        """
        res = render_video(self.vimeo_url, 500, 281).strip()
        self.assertEqual(res, self.rendered_vimeo_embed_code)
        return res

    @print_info
    def test_03_render_vimeo_responsive(self):
        """
        Test rendering of Vimeo.
        """
        res = render_video(self.vimeo_url).strip()
        self.assertEqual(res, self.rendered_vimeo_embed_code_responsive)
        return res

    @print_info
    def test_04_render_youtube(self):
        """
        Test rendering of Youtube.
        """
        res = render_video(self.youtube_url, 560, 315).strip()
        self.assertEqual(res, self.rendered_youtube_embed_code)
        return res

    @print_info
    def test_05_render_youtube_responsive(self):
        """
        Test rendering of Youtube responsive.
        """
        res = render_video(self.youtube_url).strip()
        self.assertEqual(res, self.rendered_youtube_responsive_embed_code)
        return res

    @print_info
    def test_06_register_custom_plugin(self):
        """
        Test registering of a custom plugin.
        """
        class ExamplePlugin(BaseVideoPlugin):
            """
            Example plugin.
            """
            uid = "example"
            name = "Example"
            url_pattern = "^(?P<prefix>(http\:\/\/www\.youtube\.com\/watch\?v=)|(http\:\/\/www\.youtube\.com\/v\/)|(http\:\/\/youtu\.be\/))(?P<value>[A-Za-z0-9\-=_]{11})"
            id_pattern = "^(?P<value>[A-Za-z0-9\-=_]{11})"
            thumbnail_pattern = "//img.youtube.com/vi/{0}/{1}.jpg"
            embed_code = """
            <iframe src="//www.youtube.com/embed/{video_id}" {options} frameborder="0" allowfullscreen></iframe>
            """

        plugin_registry.register(ExamplePlugin)

        assert 'example' in get_registered_plugin_uids()
        res = render_video(self.youtube_url, plugin_uid='example').strip()
        self.assertEqual(res, self.rendered_youtube_responsive_embed_code)
        return res

    @print_info
    def test_07_register_unregister(self):
        """
        Testing register/unregister.
        """
        from vishap.contrib.plugins.vimeo.vishap_plugin import VimeoPlugin

        class Example2Plugin(BaseVideoPlugin):
            """
            Example plugin.
            """
            uid = "youtube"
            name = "Example with youtube ``uid``"
            url_pattern = "^(?P<prefix>(http\:\/\/www\.youtube\.com\/watch\?v=)|(http\:\/\/www\.youtube\.com\/v\/)|(http\:\/\/youtu\.be\/))(?P<value>[A-Za-z0-9\-=_]{11})"
            id_pattern = "^(?P<value>[A-Za-z0-9\-=_]{11})"
            thumbnail_pattern = "//img.youtube.com/vi/{0}/{1}.jpg"
            embed_code = """
            <iframe src="//www.youtube.com/embed/{video_id}" {options} frameborder="0" allowfullscreen></iframe>
            """

        plugin_registry.register(Example2Plugin)
        # Since key `ru` already exists in the registry it can't be replaced (without force-register).
        res = plugin_registry.register(Example2Plugin)
        self.assertTrue(not res)

        # Now with force-register it can.
        res = plugin_registry.register(Example2Plugin, force=True)
        self.assertTrue(res)

        # Once we have it there and it's forced, we can't register another.
        res = plugin_registry.register(Example2Plugin, force=True)
        self.assertTrue(not res)

        # Unregister non-forced language pack.
        res = plugin_registry.unregister(VimeoPlugin)
        self.assertTrue(res and not VimeoPlugin.uid in get_registered_plugin_uids())

        res = plugin_registry.unregister(Example2Plugin)
        self.assertTrue(not res and Example2Plugin.uid in get_registered_plugin_uids())

    @print_info
    def test_08_override_settings(self):
        """
        Testing settings override.
        """
        def override_settings():
            return get_setting('DEBUG')

        self.assertEqual(defaults.DEBUG, override_settings())

        set_setting('DEBUG', True)

        self.assertEqual(True, override_settings())

        return override_settings()

if __name__ == '__main__':
    unittest.main()



__title__ = 'vishap.defaults'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('PLUGINS_DIR', 'CONTRIB_DIR', 'PLUGIN_MODULE_NAME', 'DEBUG')

PLUGINS_DIR = ('contrib', 'plugins')
CONTRIB_DIR =  ('contrib', 'apps')
PLUGIN_MODULE_NAME = 'vishap_plugin'

DEBUG = False



__title__ = 'vishap.utils'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('render_video', 'ensure_autodiscover', 'get_registered_plugins', 'get_registered_plugin_uids')

#from vishap.exceptions import PluginCodeError, PluginNotFound, PluginDetectionError
from vishap.base import plugin_registry
from vishap.discover import autodiscover

_ = lambda s: s

def ensure_autodiscover():
    """
    Ensures that plugins are autodiscovered.
    """
    if not plugin_registry._registry:
        autodiscover()

def get_registered_plugins():
    """
    Gets a list of registered plugins in a form if tuple (plugin name, plugin description). If not yet
    autodiscovered, autodiscovers them.

    :return list:
    """
    ensure_autodiscover()

    registered_plugins = []

    for uid, plugin in plugin_registry._registry.items():
        registered_plugins.append((uid, plugin))

    return registered_plugins

def get_registered_plugin_uids():
    """
    Gets a list of registered plugins in a form if tuple (plugin name, plugin description). If not yet
    autodiscovered, autodiscovers them.

    :return list:
    """
    ensure_autodiscover()

    return [uid for uid, plugin in plugin_registry._registry.items()]

def render_video(url, width=None, height=None, plugin_uid=None):
    """
    Renders the video.

    :param str url:
    :param int width:
    :param int height:
    :param str plugin_uid: Preferred plugin to render to video.
    :return str: Returns empty string on failure.
    """
    # Preferred plugin has been given. Try to render.
    if plugin_uid:
        plugin_cls = plugin_registry.get(plugin_uid)

        if not plugin_cls:
            # TODO: Log warnings
            return ''

        plugin = plugin_cls()
        if plugin.match(url):
            return plugin.render(url=url, width=width, height=height)
        else:
            # TODO: Log warnings
            return ''

    # No preferred plugin has been given. Detect which one shall be used and render.
    for plugin_uid, plugin_cls in get_registered_plugins():
        plugin = plugin_cls()
        if plugin.match(url):
            return plugin.render(url=url, width=width, height=height)

    return ''



__title__ = 'vishap'
__version__ = '0.1.1'
__build__ = 0x000002
__author__ = 'Artur Barseghyan'
__all__ = ('render_video', 'get_registered_plugins', 'get_registered_plugin_uids', 'ensure_autodiscover')

from vishap.utils import render_video, get_registered_plugins, get_registered_plugin_uids, ensure_autodiscover



__title__ = 'vishap.contrib.plugins.youtube.vishap_plugin'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('YoutubePlugin',)

from vishap.base import BaseVideoPlugin, plugin_registry

class YoutubePlugin(BaseVideoPlugin):
    """
    Youtube plugin.
    """
    uid = "youtube"
    name = "Youtube"
    url_pattern = "^(?P<prefix>(http\:\/\/www\.youtube\.com\/watch\?v=)|(http\:\/\/www\.youtube\.com\/v\/)|(http\:\/\/youtu\.be\/))(?P<value>[A-Za-z0-9\-=_]{11})"
    id_pattern = "^(?P<value>[A-Za-z0-9\-=_]{11})"
    thumbnail_pattern = "//img.youtube.com/vi/{0}/{1}.jpg"
    embed_code = """
    <iframe src="//www.youtube.com/embed/{video_id}" {options} frameborder="0" allowfullscreen></iframe>
    """

plugin_registry.register(YoutubePlugin)
